package sun.security.x509;

import java.util.Collection;
import java.util.Collections;
import java.util.TreeMap;
import java.util.Map;
import java.util.Enumeration;
import sun.security.util.ObjectIdentifier;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Set;
import sun.misc.HexDumpEncoder;
import java.security.cert.CRLReason;
import sun.security.util.DerOutputStream;
import java.io.IOException;
import java.security.cert.CRLException;
import sun.security.util.DerValue;
import java.math.BigInteger;
import javax.security.auth.x500.X500Principal;
import java.util.Date;
import java.security.cert.X509CRLEntry;

public class X509CRLEntryImpl extends X509CRLEntry implements Comparable<X509CRLEntryImpl>
{
    private SerialNumber serialNumber;
    private Date revocationDate;
    private CRLExtensions extensions;
    private byte[] revokedCert;
    private X500Principal certIssuer;
    private static final boolean isExplicit = false;
    
    public X509CRLEntryImpl(BigInteger bigInteger, Date revocationDate) {
        this.serialNumber = null;
        this.revocationDate = null;
        this.extensions = null;
        this.revokedCert = null;
        this.serialNumber = new SerialNumber(bigInteger);
        this.revocationDate = revocationDate;
    }
    
    public X509CRLEntryImpl(BigInteger bigInteger, Date revocationDate, CRLExtensions extensions) {
        this.serialNumber = null;
        this.revocationDate = null;
        this.extensions = null;
        this.revokedCert = null;
        this.serialNumber = new SerialNumber(bigInteger);
        this.revocationDate = revocationDate;
        this.extensions = extensions;
    }
    
    public X509CRLEntryImpl(byte[] encoding) throws CRLException {
        this.serialNumber = null;
        this.revocationDate = null;
        this.extensions = null;
        this.revokedCert = null;
        try {
            this.parse(new DerValue(encoding));
        }
        catch (IOException ex) {
            this.revokedCert = null;
            throw new CRLException("Parsing error: " + ex.toString());
        }
    }
    
    public X509CRLEntryImpl(DerValue derValue) throws CRLException {
        this.serialNumber = null;
        this.revocationDate = null;
        this.extensions = null;
        this.revokedCert = null;
        try {
            this.parse(derValue);
        }
        catch (IOException ex) {
            this.revokedCert = null;
            throw new CRLException("Parsing error: " + ex.toString());
        }
    }
    
    @Override
    public boolean hasExtensions() {
        return this.extensions != null;
    }
    
    public void encode(DerOutputStream derOutputStream) throws CRLException {
        try {
            if (this.revokedCert == null) {
                DerOutputStream derOutputStream2 = new DerOutputStream();
                this.serialNumber.encode(derOutputStream2);
                if (this.revocationDate.getTime() < 2524608000000L) {
                    derOutputStream2.putUTCTime(this.revocationDate);
                }
                else {
                    derOutputStream2.putGeneralizedTime(this.revocationDate);
                }
                if (this.extensions != null) {
                    this.extensions.encode(derOutputStream2, false);
                }
                DerOutputStream derOutputStream3 = new DerOutputStream();
                derOutputStream3.write((byte)48, derOutputStream2);
                this.revokedCert = derOutputStream3.toByteArray();
            }
            derOutputStream.write(this.revokedCert);
        }
        catch (IOException ex) {
            throw new CRLException("Encoding error: " + ex.toString());
        }
    }
    
    @Override
    public byte[] getEncoded() throws CRLException {
        return (byte[])this.getEncoded0().clone();
    }
    
    private byte[] getEncoded0() throws CRLException {
        if (this.revokedCert == null) {
            this.encode(new DerOutputStream());
        }
        return this.revokedCert;
    }
    
    @Override
    public X500Principal getCertificateIssuer() {
        return this.certIssuer;
    }
    
    void setCertificateIssuer(X500Principal x500Principal, X500Principal x500Principal2) {
        if (x500Principal.equals(x500Principal2)) {
            this.certIssuer = null;
        }
        else {
            this.certIssuer = x500Principal2;
        }
    }
    
    @Override
    public BigInteger getSerialNumber() {
        return this.serialNumber.getNumber();
    }
    
    @Override
    public Date getRevocationDate() {
        return new Date(this.revocationDate.getTime());
    }
    
    @Override
    public CRLReason getRevocationReason() {
        Extension extension = this.getExtension(PKIXExtensions.ReasonCode_Id);
        if (extension == null) {
            return null;
        }
        return ((CRLReasonCodeExtension)extension).getReasonCode();
    }
    
    public static CRLReason getRevocationReason(X509CRLEntry x509CRLEntry) {
        try {
            byte[] extensionValue = x509CRLEntry.getExtensionValue("2.5.29.21");
            if (extensionValue == null) {
                return null;
            }
            return new CRLReasonCodeExtension(Boolean.FALSE, new DerValue(extensionValue).getOctetString()).getReasonCode();
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    public Integer getReasonCode() throws IOException {
        Extension extension = this.getExtension(PKIXExtensions.ReasonCode_Id);
        if (extension == null) {
            return null;
        }
        return ((CRLReasonCodeExtension)extension).get("reason");
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.serialNumber.toString());
        sb.append("  On: " + this.revocationDate.toString());
        if (this.certIssuer != null) {
            sb.append("\n    Certificate issuer: " + this.certIssuer);
        }
        if (this.extensions != null) {
            Extension[] array = (Extension[])this.extensions.getAllExtensions().toArray(new Extension[0]);
            sb.append("\n    CRL Entry Extensions: " + array.length);
            for (int i = 0; i < array.length; ++i) {
                sb.append("\n    [" + (i + 1) + "]: ");
                Extension extension = array[i];
                try {
                    if (OIDMap.getClass(extension.getExtensionId()) == null) {
                        sb.append(extension.toString());
                        byte[] extensionValue = extension.getExtensionValue();
                        if (extensionValue != null) {
                            DerOutputStream o = new DerOutputStream();
                            o.putOctetString(extensionValue);
                            sb.append("Extension unknown: DER encoded OCTET string =\n" + new HexDumpEncoder().encodeBuffer(o.toByteArray()) + "\n");
                        }
                    }
                    else {
                        sb.append(extension.toString());
                    }
                }
                catch (Exception ex) {
                    sb.append(", Error parsing this extension");
                }
            }
        }
        sb.append("\n");
        return sb.toString();
    }
    
    @Override
    public boolean hasUnsupportedCriticalExtension() {
        return this.extensions != null && this.extensions.hasUnsupportedCriticalExtension();
    }
    
    @Override
    public Set<String> getCriticalExtensionOIDs() {
        if (this.extensions == null) {
            return null;
        }
        TreeSet<String> set = new TreeSet();
        for (Extension extension : this.extensions.getAllExtensions()) {
            if (extension.isCritical()) {
                set.add(extension.getExtensionId().toString());
            }
            else {
                continue;
            }
        }
        return set;
    }
    
    @Override
    public Set<String> getNonCriticalExtensionOIDs() {
        if (this.extensions == null) {
            return null;
        }
        TreeSet<String> set = new TreeSet();
        for (Extension extension : this.extensions.getAllExtensions()) {
            if (!extension.isCritical()) {
                set.add(extension.getExtensionId().toString());
            }
            else {
                continue;
            }
        }
        return set;
    }
    
    @Override
    public byte[] getExtensionValue(String s) {
        if (this.extensions == null) {
            return null;
        }
        try {
            String name = OIDMap.getName(new ObjectIdentifier(s));
            Extension value = null;
            if (name == null) {
                ObjectIdentifier obj = new ObjectIdentifier(s);
                Enumeration<Extension> elements = this.extensions.getElements();
                while (elements.hasMoreElements()) {
                    Extension extension = (Extension)elements.nextElement();
                    if (!extension.getExtensionId().equals(obj)) {
                        continue;
                    }
                    value = extension;
                    break;
                }
            }
            else {
                value = this.extensions.get(name);
            }
            if (value == null) {
                return null;
            }
            byte[] extensionValue = value.getExtensionValue();
            if (extensionValue == null) {
                return null;
            }
            DerOutputStream o = new DerOutputStream();
            o.putOctetString(extensionValue);
            return o.toByteArray();
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public Extension getExtension(ObjectIdentifier objectIdentifier) {
        if (this.extensions == null) {
            return null;
        }
        return this.extensions.get(OIDMap.getName(objectIdentifier));
    }
    
    private void parse(DerValue derValue) throws CRLException, IOException {
        if (derValue.tag != 48) {
            throw new CRLException("Invalid encoded RevokedCertificate, starting sequence tag missing.");
        }
        if (derValue.data.available() == 0) {
            throw new CRLException("No data encoded for RevokedCertificates");
        }
        this.revokedCert = derValue.toByteArray();
        this.serialNumber = new SerialNumber(derValue.toDerInputStream().getDerValue());
        int peekByte = derValue.data.peekByte();
        if ((byte)peekByte == 23) {
            this.revocationDate = derValue.data.getUTCTime();
        }
        else if ((byte)peekByte == 24) {
            this.revocationDate = derValue.data.getGeneralizedTime();
        }
        else {
            throw new CRLException("Invalid encoding for revocation date");
        }
        if (derValue.data.available() == 0) {
            return;
        }
        this.extensions = new CRLExtensions(derValue.toDerInputStream());
        return;
    }
    
    public static X509CRLEntryImpl toImpl(X509CRLEntry x509CRLEntry) throws CRLException {
        if ((x509CRLEntry instanceof X509CRLEntryImpl)) {
            return (X509CRLEntryImpl)x509CRLEntry;
        }
        return new X509CRLEntryImpl(x509CRLEntry.getEncoded());
    }
    
    CertificateIssuerExtension getCertificateIssuerExtension() {
        return (CertificateIssuerExtension)this.getExtension(PKIXExtensions.CertificateIssuer_Id);
    }
    
    public Map<String, java.security.cert.Extension> getExtensions() {
        if (this.extensions == null) {
            return Collections.emptyMap();
        }
        Collection<Extension> allExtensions = this.extensions.getAllExtensions();
        TreeMap<String, Extension> treeMap = new TreeMap();
        for (Extension extension : allExtensions) {
            treeMap.put(extension.getId(), extension);
        }
        return treeMap;
    }
    
    @Override
    public int compareTo(X509CRLEntryImpl x509CRLEntryImpl) {
        int compareTo = this.getSerialNumber().compareTo(x509CRLEntryImpl.getSerialNumber());
        if (compareTo != 0) {
            return compareTo;
        }
        try {
            byte[] encoded0 = this.getEncoded0();
            byte[] encoded2 = x509CRLEntryImpl.getEncoded0();
            int n = 0;
            while (n < encoded0.length && n < encoded2.length) {
                int n2 = encoded0[n] & 0xFF;
                int n3 = encoded2[n] & 0xFF;
                if (n2 != n3) {
                    return n2 - n3;
                }
                ++n;
            }
            return encoded0.length - encoded2.length;
        }
        catch (CRLException ex) {
            return -1;
        }
    }
}
